@@ -31,7 +31,6 @@
 
 struct _GsHistoryDialogPrivate
 {
-	GsApp		*app;
 	GtkSizeGroup	*sizegroup_state;
 	GtkSizeGroup	*sizegroup_timestamp;
 	GtkSizeGroup	*sizegroup_version;
@@ -60,6 +59,7 @@ gs_history_dialog_set_app (GsHistoryDialog *dialog, GsApp *app)
 	const gchar *tmp;
 	GPtrArray *history;
 	GtkBox *box;
+	GtkWidget *row;
 	GtkWidget *widget;
 	guint64 timestamp;
 	guint i;
@@ -144,6 +144,9 @@ gs_history_dialog_set_app (GsHistoryDialog *dialog, GsApp *app)
 
 		gtk_widget_show_all (GTK_WIDGET (box));
 		gtk_list_box_insert (GTK_LIST_BOX (priv->list_box), GTK_WIDGET (box), -1);
+
+		row = gtk_widget_get_parent (GTK_WIDGET (box));
+		gtk_list_box_row_set_activatable (GTK_LIST_BOX_ROW (row), FALSE);
 	}
 }
 
@@ -195,9 +198,6 @@ gs_history_dialog_finalize (GObject *object)
 	g_object_unref (priv->sizegroup_timestamp);
 	g_object_unref (priv->sizegroup_version);
 
-	if (priv->app != NULL)
-		g_object_unref (priv->app);
-
 	G_OBJECT_CLASS (gs_history_dialog_parent_class)->finalize (object);
 }
 
