@@ -49,7 +49,6 @@ G_DEFINE_TYPE_WITH_PRIVATE (GsSourcesDialog, gs_sources_dialog, GTK_TYPE_DIALOG)
 static void
 add_source (GtkListBox *listbox, GsApp *app)
 {
-	GsApp *app_tmp;
 	GtkWidget *widget;
 	GtkWidget *box;
 	GtkStyleContext *context;
@@ -74,7 +73,7 @@ add_source (GtkListBox *listbox, GsApp *app)
 
 	/* split up the types */
 	for (i = 0; i < related->len; i++) {
-		app_tmp = g_ptr_array_index (related, i);
+		GsApp *app_tmp = g_ptr_array_index (related, i);
 		switch (gs_app_get_id_kind (app_tmp)) {
 		case AS_ID_KIND_WEB_APP:
 		case AS_ID_KIND_DESKTOP:
@@ -83,6 +82,7 @@ add_source (GtkListBox *listbox, GsApp *app)
 		case AS_ID_KIND_FONT:
 		case AS_ID_KIND_CODEC:
 		case AS_ID_KIND_INPUT_METHOD:
+		case AS_ID_KIND_ADDON:
 			cnt_addon++;
 			break;
 		default:
@@ -138,6 +138,11 @@ add_source (GtkListBox *listbox, GsApp *app)
 				g_object_ref (app),
 				(GDestroyNotify) g_object_unref);
 
+	g_object_set_data_full (G_OBJECT (box),
+	                        "sort",
+	                        g_utf8_casefold (gs_app_get_name (app), -1),
+	                        g_free);
+
 	gtk_list_box_prepend (listbox, box);
 	gtk_widget_show_all (box);
 }
@@ -220,7 +225,11 @@ list_sort_func (GtkListBoxRow *a,
 		GtkListBoxRow *b,
 		gpointer user_data)
 {
-	return a < b;
+	GObject *o1 = G_OBJECT (gtk_bin_get_child (GTK_BIN (a)));
+	GObject *o2 = G_OBJECT (gtk_bin_get_child (GTK_BIN (b)));
+	const gchar *key1 = g_object_get_data (o1, "sort");
+	const gchar *key2 = g_object_get_data (o2, "sort");
+	return g_strcmp0 (key1, key2);
 }
 
 static void
@@ -228,6 +237,7 @@ add_app (GtkListBox *listbox, GsApp *app)
 {
 	GtkWidget *box;
 	GtkWidget *widget;
+	GtkWidget *row;
 
 	box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
 	gtk_widget_set_margin_top (box, 12);
@@ -239,9 +249,17 @@ add_app (GtkListBox *listbox, GsApp *app)
 	gtk_widget_set_halign (widget, GTK_ALIGN_START);
 	gtk_box_pack_start (GTK_BOX (box), widget, FALSE, FALSE, 0);
 
+	g_object_set_data_full (G_OBJECT (box),
+	                        "sort",
+	                        g_utf8_casefold (gs_app_get_name (app), -1),
+	                        g_free);
+
 	gtk_list_box_prepend (listbox, box);
 	gtk_widget_show (widget);
 	gtk_widget_show (box);
+
+	row = gtk_widget_get_parent (box);
+	gtk_list_box_row_set_activatable (GTK_LIST_BOX_ROW (row), FALSE);
 }
 
 static void
@@ -264,11 +282,11 @@ list_row_activated_cb (GtkListBox *list_box,
 					 "GsShell::app"));
 	related = gs_app_get_related (app);
 	for (i = 0; i < related->len; i++) {
-		app = g_ptr_array_index (related, i);
-		switch (gs_app_get_kind (app)) {
+		GsApp *app_tmp = g_ptr_array_index (related, i);
+		switch (gs_app_get_kind (app_tmp)) {
 		case GS_APP_KIND_NORMAL:
 		case GS_APP_KIND_SYSTEM:
-			add_app (GTK_LIST_BOX (priv->listbox_apps), app);
+			add_app (GTK_LIST_BOX (priv->listbox_apps), app_tmp);
 			cnt_apps++;
 			break;
 		default:
@@ -335,6 +353,7 @@ remove_button_cb (GtkWidget *widget, GsSourcesDialog *dialog)
 
 	/* remove source */
 	app = GS_APP (g_object_get_data (G_OBJECT (priv->stack), "GsShell::app"));
+	g_debug ("removing source '%s'", gs_app_get_name (app));
 	gs_plugin_loader_app_action_async (priv->plugin_loader,
 					   app,
 					   GS_PLUGIN_LOADER_ACTION_REMOVE,
@@ -343,12 +362,21 @@ remove_button_cb (GtkWidget *widget, GsSourcesDialog *dialog)
 					   dialog);
 }
 
+static void
+updates_changed_cb (GsPluginLoader *plugin_loader,
+                    GsSourcesDialog *dialog)
+{
+	reload_sources (dialog);
+}
+
 static void
 set_plugin_loader (GsSourcesDialog *dialog, GsPluginLoader *plugin_loader)
 {
 	GsSourcesDialogPrivate *priv = gs_sources_dialog_get_instance_private (dialog);
 
 	priv->plugin_loader = g_object_ref (plugin_loader);
+	g_signal_connect (priv->plugin_loader, "updates-changed",
+	                  G_CALLBACK (updates_changed_cb), dialog);
 }
 
 static void
@@ -357,7 +385,10 @@ gs_sources_dialog_dispose (GObject *object)
 	GsSourcesDialog *dialog = GS_SOURCES_DIALOG (object);
 	GsSourcesDialogPrivate *priv = gs_sources_dialog_get_instance_private (dialog);
 
-	g_clear_object (&priv->plugin_loader);
+	if (priv->plugin_loader != NULL) {
+		g_signal_handlers_disconnect_by_func (priv->plugin_loader, updates_changed_cb, dialog);
+		g_clear_object (&priv->plugin_loader);
+	}
 
 	if (priv->cancellable != NULL) {
 		g_cancellable_cancel (priv->cancellable);
