@@ -397,7 +397,7 @@ refresh_cache (GsUpdateMonitor *monitor)
 
 	g_debug ("Refreshing cache");
 	pk_client_refresh_cache_async (PK_CLIENT (monitor->task),
-				       TRUE,
+				       FALSE,
 				       monitor->cancellable,
 				       NULL, NULL,
 				       refresh_cache_finished_cb,
@@ -521,6 +521,7 @@ gs_update_monitor_init (GsUpdateMonitor *monitor)
 		      "background", TRUE,
 		      "interactive", FALSE,
 		      "only-download", TRUE,
+		      "cache-age", 60 * 60 * 24,
 		      NULL);
 
 	monitor->network_available = FALSE;
