@@ -32,6 +32,7 @@
 
 struct GsPluginPrivate {
 	AsStore			*store;
+	GMutex			 store_mutex;
 	gchar			*locale;
 	gsize			 done_init;
 	gboolean		 has_hi_dpi_support;
@@ -88,6 +89,7 @@ void
 gs_plugin_initialize (GsPlugin *plugin)
 {
 	plugin->priv = GS_PLUGIN_GET_PRIVATE (GsPluginPrivate);
+	g_mutex_init (&plugin->priv->store_mutex);
 	plugin->priv->store = as_store_new ();
 	g_signal_connect (plugin->priv->store, "changed",
 			  G_CALLBACK (gs_plugin_appstream_store_changed_cb),
@@ -120,6 +122,7 @@ gs_plugin_destroy (GsPlugin *plugin)
 {
 	g_free (plugin->priv->locale);
 	g_object_unref (plugin->priv->store);
+	g_mutex_clear (&plugin->priv->store_mutex);
 }
 
 /**
@@ -184,6 +187,8 @@ gs_plugin_startup (GsPlugin *plugin, GError **error)
 	guint i;
 	_cleanup_hashtable_unref_ GHashTable *origins = NULL;
 
+	g_mutex_lock (&plugin->priv->store_mutex);
+
 	/* clear all existing applications if the store was invalidated */
 	as_store_remove_all (plugin->priv->store);
 
@@ -244,6 +249,7 @@ gs_plugin_startup (GsPlugin *plugin, GError **error)
 		}
 	}
 out:
+	g_mutex_unlock (&plugin->priv->store_mutex);
 	gs_profile_stop (plugin->profile, "appstream::startup");
 	return ret;
 }
@@ -666,6 +672,8 @@ gs_plugin_refine_from_id (GsPlugin *plugin,
 	gboolean ret = TRUE;
 	AsApp *item = NULL;
 
+	g_mutex_lock (&plugin->priv->store_mutex);
+
 	/* find anything that matches the ID */
 	id = gs_app_get_id (app);
 	if (id == NULL)
@@ -679,6 +687,7 @@ gs_plugin_refine_from_id (GsPlugin *plugin,
 	if (!ret)
 		goto out;
 out:
+	g_mutex_unlock (&plugin->priv->store_mutex);
 	*found = (item != NULL);
 	return ret;
 }
@@ -693,9 +702,12 @@ gs_plugin_refine_from_pkgname (GsPlugin *plugin,
 {
 	AsApp *item = NULL;
 	GPtrArray *sources;
+	gboolean ret = TRUE;
 	const gchar *pkgname;
 	guint i;
 
+	g_mutex_lock (&plugin->priv->store_mutex);
+
 	/* find anything that matches the ID */
 	sources = gs_app_get_sources (app);
 	for (i = 0; i < sources->len && item == NULL; i++) {
@@ -708,10 +720,13 @@ gs_plugin_refine_from_pkgname (GsPlugin *plugin,
 
 	/* nothing found */
 	if (item == NULL)
-		return TRUE;
+		goto out;
 
 	/* set new properties */
-	return gs_plugin_refine_item (plugin, app, item, error);
+	ret = gs_plugin_refine_item (plugin, app, item, error);
+out:
+	g_mutex_unlock (&plugin->priv->store_mutex);
+	return ret;
 }
 
 /**
@@ -734,7 +749,7 @@ gs_plugin_refine (GsPlugin *plugin,
 		ret = gs_plugin_startup (plugin, error);
 		g_once_init_leave (&plugin->priv->done_init, TRUE);
 		if (!ret)
-			goto out;
+			return FALSE;
 	}
 
 	gs_profile_start (plugin->profile, "appstream::refine");
@@ -780,11 +795,12 @@ gs_plugin_add_category_apps (GsPlugin *plugin,
 		ret = gs_plugin_startup (plugin, error);
 		g_once_init_leave (&plugin->priv->done_init, TRUE);
 		if (!ret)
-			goto out;
+			return FALSE;
 	}
 
 	/* get the two search terms */
 	gs_profile_start (plugin->profile, "appstream::add-category-apps");
+	g_mutex_lock (&plugin->priv->store_mutex);
 	search_id1 = gs_category_get_id (category);
 	parent = gs_category_get_parent (category);
 	if (parent != NULL)
@@ -816,6 +832,7 @@ gs_plugin_add_category_apps (GsPlugin *plugin,
 		gs_plugin_add_app (list, app);
 	}
 out:
+	g_mutex_unlock (&plugin->priv->store_mutex);
 	gs_profile_stop (plugin->profile, "appstream::add-category-apps");
 	return ret;
 }
@@ -914,11 +931,12 @@ gs_plugin_add_search (GsPlugin *plugin,
 		ret = gs_plugin_startup (plugin, error);
 		g_once_init_leave (&plugin->priv->done_init, TRUE);
 		if (!ret)
-			goto out;
+			return FALSE;
 	}
 
 	/* search categories for the search term */
 	gs_profile_start (plugin->profile, "appstream::search");
+	g_mutex_lock (&plugin->priv->store_mutex);
 	array = as_store_get_apps (plugin->priv->store);
 	for (i = 0; i < array->len; i++) {
 		if (g_cancellable_set_error_if_cancelled (cancellable, error))
@@ -930,6 +948,7 @@ gs_plugin_add_search (GsPlugin *plugin,
 			goto out;
 	}
 out:
+	g_mutex_unlock (&plugin->priv->store_mutex);
 	gs_profile_stop (plugin->profile, "appstream::search");
 	return ret;
 }
@@ -953,11 +972,12 @@ gs_plugin_add_installed (GsPlugin *plugin,
 		ret = gs_plugin_startup (plugin, error);
 		g_once_init_leave (&plugin->priv->done_init, TRUE);
 		if (!ret)
-			goto out;
+			return FALSE;
 	}
 
 	/* search categories for the search term */
 	gs_profile_start (plugin->profile, "appstream::add_installed");
+	g_mutex_lock (&plugin->priv->store_mutex);
 	array = as_store_get_apps (plugin->priv->store);
 	for (i = 0; i < array->len; i++) {
 		item = g_ptr_array_index (array, i);
@@ -971,6 +991,7 @@ gs_plugin_add_installed (GsPlugin *plugin,
 		}
 	}
 out:
+	g_mutex_unlock (&plugin->priv->store_mutex);
 	gs_profile_stop (plugin->profile, "appstream::add_installed");
 	return ret;
 }
@@ -1039,11 +1060,12 @@ gs_plugin_add_categories (GsPlugin *plugin,
 		ret = gs_plugin_startup (plugin, error);
 		g_once_init_leave (&plugin->priv->done_init, TRUE);
 		if (!ret)
-			goto out;
+			return FALSE;
 	}
 
 	/* find out how many packages are in each category */
 	gs_profile_start (plugin->profile, "appstream::add-categories");
+	g_mutex_lock (&plugin->priv->store_mutex);
 	array = as_store_get_apps (plugin->priv->store);
 	for (i = 0; i < array->len; i++) {
 		app = g_ptr_array_index (array, i);
@@ -1053,7 +1075,7 @@ gs_plugin_add_categories (GsPlugin *plugin,
 			continue;
 		gs_plugin_add_categories_for_app (*list, app);
 	}
-out:
+	g_mutex_unlock (&plugin->priv->store_mutex);
 	gs_profile_stop (plugin->profile, "appstream::add-categories");
 	return ret;
 }
@@ -1305,9 +1327,10 @@ gs_plugin_add_popular (GsPlugin *plugin,
 		ret = gs_plugin_startup (plugin, error);
 		g_once_init_leave (&plugin->priv->done_init, TRUE);
 		if (!ret)
-			goto out;
+			return FALSE;
 	}
 	gs_profile_start (plugin->profile, "appstream::add_popular");
+	g_mutex_lock (&plugin->priv->store_mutex);
 
 	/* get already installed applications */
 	ignore_apps = g_hash_table_new (g_str_hash, g_str_equal);
@@ -1348,6 +1371,7 @@ gs_plugin_add_popular (GsPlugin *plugin,
 	}
 
 out:
+	g_mutex_unlock (&plugin->priv->store_mutex);
 	gs_profile_stop (plugin->profile, "appstream::add_popular");
 	return ret;
 }
